@@ -40,7 +40,7 @@
 #include <errno.h>
 #include <time.h>
 #include <libpq-fe.h>
-#include "dirty.h"
+#include "conf.h"
 #include "syslogd-types.h"
 #include "srUtils.h"
 #include "template.h"
