@@ -23,8 +23,6 @@
 #ifndef INCLUDED_DATETIME_H
 #define INCLUDED_DATETIME_H
 
-#include "datetime.h"
-
 /* TODO: define error codes */
 #define NO_ERRCODE -1
 
@@ -36,20 +34,22 @@ typedef struct datetime_s {
 /* interfaces */
 BEGINinterface(datetime) /* name must also be changed in ENDinterface macro! */
 	void (*getCurrTime)(struct syslogTime *t, time_t *ttSeconds);
-	rsRetVal (*ParseTIMESTAMP3339)(struct syslogTime *pTime, uchar** ppszTS, int *);
-	rsRetVal (*ParseTIMESTAMP3164)(struct syslogTime *pTime, uchar** pszTS, int *);
-	int (*formatTimestampToMySQL)(struct syslogTime *ts, char* pDst, size_t iLenDst);
-	int (*formatTimestampToPgSQL)(struct syslogTime *ts, char *pDst, size_t iLenDst);
-	int (*formatTimestamp3339)(struct syslogTime *ts, char* pBuf, size_t iLenBuf);
-	int (*formatTimestamp3164)(struct syslogTime *ts, char* pBuf, size_t iLenBuf);
-	int (*formatTimestampSecFrac)(struct syslogTime *ts, char* pBuf, size_t iLenBuf);
+	rsRetVal (*ParseTIMESTAMP3339)(struct syslogTime *pTime, uchar** ppszTS, int*);
+	rsRetVal (*ParseTIMESTAMP3164)(struct syslogTime *pTime, uchar** pszTS, int*);
+	int (*formatTimestampToMySQL)(struct syslogTime *ts, char* pDst);
+	int (*formatTimestampToPgSQL)(struct syslogTime *ts, char *pDst);
+	int (*formatTimestamp3339)(struct syslogTime *ts, char* pBuf);
+	int (*formatTimestamp3164)(struct syslogTime *ts, char* pBuf, int);
+	int (*formatTimestampSecFrac)(struct syslogTime *ts, char* pBuf);
 ENDinterface(datetime)
-#define datetimeCURR_IF_VERSION 2 /* increment whenever you change the interface structure! */
+#define datetimeCURR_IF_VERSION 4 /* increment whenever you change the interface structure! */
 /* interface changes:
  * 1 - initial version
  * 2 - not compatible to 1 - bugfix required ParseTIMESTAMP3164 to accept char ** as
  *     last parameter. Did not try to remain compatible as this is not something any
  *     third-party module should call. -- rgerhards, 2008.-09-12
+ * 3 - taken by v5 branch!
+ * 4 - formatTimestamp3164 takes a third int parameter
  */
 
 /* prototypes */
